import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import javax.persistence.Entity;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Id;
import javax.persistence.Persistence;
import javax.persistence.Tuple;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Join;
import javax.persistence.criteria.Root;

public class Main {

	public static void main(String[] args) {
		
		EntityManagerFactory emf = Persistence.createEntityManagerFactory("$objectdb/testdb.odb");
		
		EntityManager em = emf.createEntityManager();
		EntityClass parent = creteData(em);
		em.close();
		
		em = emf.createEntityManager();
		
		CriteriaBuilder cb = em.getCriteriaBuilder();
		CriteriaQuery<Tuple> tupleQuery = cb.createTupleQuery();
		Root<EntityChild> root = tupleQuery.from(EntityChild.class);
		Join<EntityChild, EntityClass> joinParent = root.join("parent");
		
		//uncomment line below to get yet another exception
		//root.alias("r");
		
		//uncomment line below to make things work
		//joinParent.alias("p");
		
		tupleQuery.multiselect(root, joinParent);
		
		TypedQuery<Tuple> query = em.createQuery(tupleQuery);
		List<Tuple> resultList = query.getResultList();
		
		Tuple tuple = resultList.get(0);

		EntityChild e3 = tuple.get(root); //OK
		Object untyped = tuple.get(joinParent); //INVALID TUPLE ELEMENT RETURNED!
		EntityClass c3 = tuple.get(joinParent); //FAILS!
		
		em.close();
		System.out.println("Done");
	}

	private static EntityClass creteData(EntityManager em) {

		em.getTransaction().begin();
		EntityClass e1 = new EntityClass();
		e1 = em.merge(e1);
		e1.setSomeDataField("test data");
		
		EntityChild c1 = em.merge(new EntityChild());
		c1.setParent(e1);
		e1.getChildren().add(c1);
		
		EntityChild c2 = em.merge(new EntityChild());
		c2.setParent(e1);
		e1.getChildren().add(c2);
		
		em.getTransaction().commit();
		return e1;
	}

	@Entity
	public static class EntityClass {

		private String id = UUID.randomUUID().toString();
		private String someDataField;
		private List<EntityChild> children = new ArrayList<>();
		
		@Id
		public String getId() { return id; }
		public void   setId(String id) { this.id = id;}
		
		public String getSomeDataField() {
			return someDataField;
		}
		public void setSomeDataField(String someDataField) {
			this.someDataField = someDataField;
		}
		public List<EntityChild> getChildren() {
			return children;
		}
		public void setChildren(List<EntityChild> children) {
			this.children = children;
		}
	}

	@Entity
	public static class EntityChild {

		private String id = UUID.randomUUID().toString();
		private EntityClass parent;
		
		@Id
		public String getId() { return id; }
		public void   setId(String id) { this.id = id;}
		public EntityClass getParent() {
			return parent;
		}
		public void setParent(EntityClass parent) {
			this.parent = parent;
		}
	}
	
}
